package uk.co.hcsoftware.images.gui;

import javax.media.jai.PlanarImage;
import java.awt.image.DataBuffer;
import java.awt.image.SampleModel;
import java.io.File;
import java.util.*;
import java.util.logging.Logger;
import java.util.logging.Level;


/**
 * Title: ImageCache
 * Copyright:    Copyright (c) 09-Jan-2006
 *
 * @author spurlinn
 */
public class ImageCache {
    public static final Logger log = Logger.getLogger(ImageCache.class.getName());
    private Map<File,CacheRecord> store = new HashMap<File,CacheRecord>();
  //  private final long maxBytes;
    private int index=0;
    private int maxSize;

    public ImageCache(int cacheSize) {
        this.maxSize = cacheSize;
    }

    public ImageCache() {
      //  this(1000*1000*2);//2Mb
    }

    public void setAllPriorities(int priority) {
        for (Iterator<CacheRecord> iterator = store.values().iterator(); iterator.hasNext();) {
            CacheRecord cacheRecord = iterator.next();
            cacheRecord.setPriority(priority);
        }
    }

    private class CacheRecord{
        private File key;
        private PlanarImage pi;
        private int index;
        private int priority;
        public CacheRecord(File key, PlanarImage pi,int priority) {
            this.key = key;
            this.pi = pi;
            this.priority = priority;
            //bytes=calculateBytes(pi);
            //these are vague guesstimates - no idea how much the overhead is really..
            this.index=ImageCache.this.index++;
        }

        public void setPriority(int priority) {
            this.priority = priority;
        }

        public int getPriority() {
            return priority;
        }

        public int getIndex() {
            return index;
        }

        public File getKey() {
            return key;
        }

        public PlanarImage getImage() {
            return pi;
        }

        public String toString() {
            return "file="+key+" pri="+priority+" index="+index;
        }
    }
    public boolean isCached(final File f2) {
        return store.containsKey(f2);
    }
    public int getMaxSize(){
        return store.size();
    }
    /**
     * caches image with (default) priority of 5
     * @param pi
     * @param f2
     */
    public void put(PlanarImage pi, final File f2) {
        put(pi,f2,5);
    }
    /**
     *
     * @param pi
     * @param f2
     * @param priority integer indicating how important this record is relative to other records for the
     * purpose of determining when it should be removed from the cache, higher being more important
     */
    public void put(PlanarImage pi, final File f2, int priority) {
        final CacheRecord cr = new CacheRecord(f2,pi,priority);

        if (store.size()>=maxSize) { //clear some space
            LinkedList<CacheRecord> records = new LinkedList<CacheRecord>(store.values());
            //sort by priority, age(reverse index)
            Collections.sort(records,new Comparator<CacheRecord>() {
                public int compare(CacheRecord cr, CacheRecord cr1) {
                    int i=cr.getPriority()-cr1.getPriority();
                    if(i==0){
                       i=(cr.getIndex()-cr1.getIndex());
                    }
                    return i;
                }
            });
            if(records.size()!=store.size()) log.warning("sorting truncated list! list="+records);
          //  while(records.size()>0 && getUsedBytes(records)+cr.getBytes()>maxBytes){
                if (log.isLoggable(Level.FINER)) {
                    log.finer("cachelist="+records);
                }
                //lose the first, unless it is higher priority than the record to be cached
                final CacheRecord cacheRecord =records.getFirst();
                if (cacheRecord.getPriority()<=cr.getPriority()) {
                    records.removeFirst();
                    log.finer("clearing space by removing cacheRecord="+cacheRecord);
                    final File key = cacheRecord.getKey();
                    store.remove(key);
                } else {
                    log.fine("ignoring record="+cr+" as all entries in cache have higher priority");
                }
           // }
        }

        store.put(f2,cr);
        log.fine("encached record="+cr+ " objects="+getMaxSize());
    }
//    public long getUsedBytes(Collection<CacheRecord> records) {
//        long used=0;
//        for (Iterator<CacheRecord> iterator = records.iterator(); iterator.hasNext();) {
//            CacheRecord cacheRecord = iterator.next();
//            used+=cacheRecord.getBytes();
//        }
//        return used;
//    }
//    public long getUsedBytes() {
//        return getUsedBytes(store.values());
//    }

    public PlanarImage get(File file) {
        final CacheRecord cr = store.get(file);
        return cr==null?null:cr.getImage();
    }

    public void clear() {
        log.fine("clear store");
        store.clear();
    }
    /**
     *
     * @param pi
     * @return approximate value for the number of bytes that it takes to store this image on the heap
     */
    private long calculateBytes(PlanarImage pi){
        SampleModel sm = pi.getSampleModel();
        final int bytesPerPixelPerBand;
        switch(sm.getDataType()){
            case DataBuffer.TYPE_BYTE:
            case DataBuffer.TYPE_SHORT:
            case DataBuffer.TYPE_USHORT:
            case DataBuffer.TYPE_INT:
                bytesPerPixelPerBand=4;
                break;
            default: bytesPerPixelPerBand=8;
        }
        return bytesPerPixelPerBand * sm.getNumBands() * pi.getWidth() * pi.getHeight();
    }

}
