package uk.co.hcsoftware.images.gui;

import com.sun.media.jai.widget.DisplayJAI;
import ns.TaskTimer;
import uk.co.hcsoftware.images.UsageException;
import uk.co.hcsoftware.images.tools.ImageOperations;
import uk.co.hcsoftware.yago.Option;
import uk.co.hcsoftware.yago.parsers.GnuParser;
import uk.co.hcsoftware.yago.util.DefaultArgReaders;

import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Timer;
import java.util.logging.Logger;
import java.util.prefs.Preferences;

/**
 * image viewer app for flicking through images/thumbnails and classifying them.
 *
 * Tried windows native libs and didn't seem any faster to load/scale/display..
 * was taking ~937ms to display ~1.5mb png files
 * java version was taking ~1s
 *   names:
 *
 *
 */
public class ImageMonkey extends JFrame{
    private static final Logger log = Logger.getLogger(ImageMonkey.class.getName());
    private static final Preferences prefs = Preferences.userNodeForPackage(ImageMonkey.class);

    private static final int DEFAULT_CACHE_SIZE = 5;
    //  private static final Option nocache = new Option("nocache",false,"don't cache images in memory");
    private static final Option<Integer> cacheCount =  new Option<Integer>("cachesize",false,'c',"cachesize",DefaultArgReaders.createIntReader(),"Number of images to cache in memory. Default is "+DEFAULT_CACHE_SIZE+", 0 means disable caching. You may run out of memory if this is too high.");

    private DisplayJAI displayPanel = new DisplayJAI();
    ImageOperations io = new ImageOperations();
    private int index=0;

    private Map<File,PlanarImage> cache;


    private AbstractAction nextAction = new AbstractAction() {
        public void actionPerformed(ActionEvent event) {
            final int i = getNextIndex(index);
            showImage(i, 1);
        }
    };

    private int getNextIndex(int currentIndex) {
        return currentIndex+1>=imageFiles.size()?0:currentIndex+1;
    }

    private AbstractAction previousAction = new AbstractAction() {
        public void actionPerformed(ActionEvent event) {
            final int i = getPreviousIndex(index);
            showImage(i, 1);
        }
    };

    private int getPreviousIndex(int currentIndex) {
        return currentIndex==0?imageFiles.size()-1:currentIndex-1;
    }

    public static void main(String args[]) {
        GnuParser gp=new GnuParser(cacheCount);
        gp.parseOrDie("ImageMonkey",args);
        try {
            List<String> arguments =gp.getExtraArgs();
            if(arguments.size()==0 ){
                throw new UsageException("0 args");
            }
            if(Arrays.asList(new String[]{"--help","help","h","-h"}).contains(arguments.get(0))){
                throw new UsageException("invalid args");
            }
            List<File> imageFiles = new ArrayList<File>(arguments.size());
            for(String arg:arguments){
                imageFiles.add(new File(arg));
            }
            String s = System.getProperty("sun.java2d.opengl");
            if(s==null){
                System.setProperty("sun.java2d.opengl","True");
            }
            new ImageMonkey(imageFiles);
            return;
        } catch(UsageException e){
            ;
        }catch (Exception e) {
            e.printStackTrace();
        }
        String s = gp.getUsage(ImageMonkey.class.getName()+" imagefile1 [imagefilen..]",getLongDescription());
        System.out.println(s);
    }
    private static String getLongDescription() {
        return //ImageMonkey.class.getName()+" imagefile1 [imagefilen..]" +
                "\n\tdisplays a window with the first image." +
                "\n\timages are scaled to fit the window" +

                "\n\n Controls:"+
                "\n\tPgDn,n,SPACE:\tnext image" +
                "\n\tPgUp,p,DEL:\t\tprev image" +
                "\n\tHOME:\t\t\tfirst image" +
                "\n\tEND:\t\t\tlast image" +
                "\n"+
                "\n\tr:\t\t\treload current image" +
                "\n"+
                // "(textfiles holding 1 filename per line, created in workingdir, called imagenames_${keyname})"+
                "\n\tF1-F5:\t\t\tappend filename to quickfile" +
                "\n\tSHIFT+[F1-F5]:\tremove curr imagename from quickfile" +
                "\n\tCTRL+[F1-F5]:\tmove curr imagename to quickfile"
                ;
    }

    final List<File> imageFiles;
    private float zoomFactor=1f;
    //todo: cropping: should crop the _original file_ and reload (not the scaled version)
    private boolean useCache(){
        return cacheSize>0;
    }
    private int cacheSize=DEFAULT_CACHE_SIZE;
    public ImageMonkey(final List<File> imageFiles) {
        super("imageviewer");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.imageFiles=imageFiles;
        // getContentPane().setLayout(new BorderLayout(2,0));
        add(new JScrollPane(displayPanel));

        setupKeys();

        if(cacheCount.isSet()){
            cacheSize=cacheCount.getArgValue();
        }

        if(useCache()){
            log.fine("allocating cache of "+cacheSize+" images");
          //  ic = new ImageCache(maxSize);
            cache = Collections.synchronizedMap(new LinkedHashMap<File, PlanarImage>(cacheSize+1,1f,true){ //1 to hold the added entry since removeEldestEntry is called _after_ add / addAll
                public boolean removeEldestEntry(Map.Entry<File, PlanarImage> entry) {
                    final boolean b = cache.size()>cacheSize;
                    if (b) {
                        log.fine("remove eldest entry from cache currsize="+cache.size());
                    }
                    return b;
                }
            });
        }

        //restore size
        int h=prefs.getInt("frameheight",600);
        int w=prefs.getInt("framewidth",800);
        log.finer("restore framesize prop="+w+" h="+h);
        if(w<10 || h<10){
            log.warning("unsensible framesizes, using defaults prop="+w+" h="+h);
            w=800; h=600;
        }
        setSize(w,h);

        addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent e) {
                final int h = getHeight();
                final int w = getWidth();
                log.finer("saving framesize prop="+w+" h="+h);
                prefs.putInt("frameheight",h);
                prefs.putInt("framewidth",w);
                log.info("component resized: repainting");
                repaint();
                clearCache();
            }
        });


        setVisible(true);
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
                showImage(0, 1);
            }
        });
    }



    private void setupKeys() {
        final InputMap inputMap = displayPanel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        final ActionMap actionMap = displayPanel.getActionMap();

        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_DOWN,0),"next");
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_N,0),"next");
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0),"next");

        actionMap.put("next",nextAction);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_UP,0),"previous");
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,0),"previous");
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_P,0),"previous");


        actionMap.put("previous",previousAction);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_HOME,0),"first");
        actionMap.put("first",new AbstractAction() {
            public void actionPerformed(ActionEvent event) {
                showImage(0, 1);
            }
        });
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_END,0),"last");
        actionMap.put("last",new AbstractAction() {
            public void actionPerformed(ActionEvent event) {
                showImage(imageFiles.size()-1, 1);
            }
        });
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_R,0),"reload");
        actionMap.put("reload",new AbstractAction() {
            public void actionPerformed(ActionEvent event) {
                showImage(index, 1);
            }
        });

        setupQuickfilesKeys(inputMap, actionMap, imageFiles);

        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_Z,0),"zoomin");
        actionMap.put("zoomin",new AbstractAction() {
            public void actionPerformed(ActionEvent event) {
                showImage(index, 1+zoomFactor);
            }
        });
        setupZoomKeys(inputMap, actionMap);
        /*inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_MINUS,0),"zoomout");
        actionMap.put("zoomout",new AbstractAction() {
            public void actionPerformed(ActionEvent event) {
                log.log(Level.INFO,"zoomout");
                showImage(index, zoomFactor-1);
            }
        });*/
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_O,0),"noscale");
        actionMap.put("noscale",new AbstractAction() {
            public void actionPerformed(ActionEvent event) {
                showImage(index, 0);
            }
        });


        //image edit actions
//        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_R,KeyEvent.CTRL_MASK),"rotate");
//        actionMap.put("rotate",new AbstractAction() {
//            public void actionPerformed(ActionEvent event) {
//                String angle = JOptionPane.showInputDialog(ImageMonkey.this
//                        ,"enter angle between -1 and 1 to rotate"
//                ,"90");
//                if(angle!=null){
//                    final String title=getTitle();
//                    try {
//                        float angleDegrees = Float.parseFloat(angle);
//                        float angleRads = (float)Math.toRadians(angleDegrees);
//                        setTitle("rotating "+angleDegrees+" degrees...");
//
//                        TaskTimer tt = new TaskTimer();
//                        File file = imageFiles.get(index);
//                        String fileName=file.getName();
//                        log.log(Level.INFO,"Creating operation to load image from '" + fileName+"'");
//                        RenderedOp img =  JAI.create("fileload", file.getCanonicalPath());
//                        // h & w need to be height and width minus chrome
//                        PlanarImage pi = io.rotate(img,angleRads);
//                        setImage(pi, 0);
//                        log.log(Level.INFO,tt.stopString()+" to display "+fileName);
//                        setTitle(getDefaultTitle()+" rotated "+angleDegrees+" degrees");
//                    } catch (NumberFormatException e) {
//                        setTitle(title);
//                        log.log(Level.INFO,"cannot parse "+angle+" into float.. ignoring");
//                    } catch (IOException e) {
//                        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
//                    }
//                }
//            }
//        });

    }
    Timer t = new Timer(true);
    private volatile int timerTask=0;
    private void enCache(PlanarImage pi, final File f2, final int currentIndex) {
        if(!useCache()) return;
        if(!cache.containsKey(f2)){
            cache.put(f2,pi);
            log.info("cache "+f2);
        }

        cache.get(f2);//make sure this is the mostrecently accessed

        //launch a new timertask to do the preloading, this should continue while no further tasks are scheduled
        t.schedule(new TimerTask() {
            public void run() {
                final int myId=(++timerTask);
                log.fine("incremented timerId="+timerTask);
                synchronized(t){
                    final int toCache = cacheSize-2;  //1 for the current and 1 for the previous
                    log.fine("executing cacheworker id="+myId+" toCache="+toCache+ " timerId="+timerTask);
                    TaskTimer tt = new TaskTimer();
                    int cached=0;
                    for(int i=0,imageIndex=currentIndex; i<toCache;i++){
                        if(myId!=timerTask) break;
                        try {
                            imageIndex = getNextIndex(imageIndex);
                            final File f2 = imageFiles.get(imageIndex);
                            if(!cache.containsKey(f2)){
                                PlanarImage pi = fetchScaledImage(f2,1f);
                                log.info("pre-caching "+f2+ " workerid="+myId);
                                cache.put(f2,pi);
                                cached++;
                            }
                            else log.fine("no need to pre-cache "+f2+" as already in cache");
                        } catch (IOException e) {
                            log.warning("couldn't cache image for file="+f2);
                        }
                    }
                    if (cached>0) {
                        log.info(tt.stopString()+" to pre-cache "+cached+" nextmost images");
                    }
                }
            }
        }, 0);
    }

    private boolean isCached(final File f2) {
        if(!useCache()) return false;
        return cache.containsKey(f2);
    }

    private PlanarImage fetchFromCache(File file) {
        if(!useCache()) throw new RuntimeException("nocache is set, but file requested from cache!"){};
        return cache.get(file);
        //return ic.get(file);
    }

    private void clearCache() {
        if(!useCache()) return;
        cache.clear();
    }

    private void setupZoomKeys(final InputMap inputMap, final ActionMap actionMap) {
        for(int i=0;i<10;i++){
            inputMap.put(KeyStroke.getKeyStroke(0x30+i,0),"zoom"+i);
            final int i1 = i;
            actionMap.put("zoom"+i,new AbstractAction() {
                public void actionPerformed(ActionEvent event) {
                    showImage(index, i1+1);
                }
            });
        }
    }


    private void setupQuickfilesKeys(final InputMap inputMap, final ActionMap actionMap, final List<File> imageFiles) {
        for(final QuickFiles qf: QuickFiles.values()){
            //add actions
            String actionname = "addto_"+qf.getFile().getName();
            KeyStroke keyStroke = KeyStroke.getKeyStroke(qf.getKey(),0);
            inputMap.put(keyStroke,actionname);
            actionMap.put(actionname,new AbstractAction() {
                public void actionPerformed(ActionEvent event) {
                    try {
                        File currF = imageFiles.get(index);
                        final String filename = currF.getName();
                        qf.addName(filename);
                        setTitle(getDefaultTitle()+" appended to "+qf.getFile().getName()+" "+df.format(new Date()));
                    } catch (IOException e) {
                        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        JOptionPane.showMessageDialog(ImageMonkey.this,"cannot write name to file: \n"+e.getMessage(),"add name to file",JOptionPane.ERROR_MESSAGE);
                    }
                }
            });

            //remove actions
            actionname = "removefrom_"+qf.getFile().getName();
            keyStroke = KeyStroke.getKeyStroke(qf.getKey(),KeyEvent.SHIFT_MASK);
            inputMap.put(keyStroke,actionname);
            actionMap.put(actionname,new AbstractAction() {
                public void actionPerformed(ActionEvent event) {
                    if(!qf.getFile().exists()){
                        setTitle(getDefaultTitle()+" no such file:  "+qf.getFile().getName()+" "+df.format(new Date()));
                        return;
                    }
                    try {
                        File currF = imageFiles.get(index);
                        final String filename = currF.getName();
                        int linesRemoved = qf.removeName(filename);
                        setTitle(getDefaultTitle()+" removed from "
                                +qf.getFile().getName()+" (-"+linesRemoved+" lines) "
                                +df.format(new Date()));
                    } catch (IOException e) {
                        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        JOptionPane.showMessageDialog(ImageMonkey.this,"cannot remove name from file: \n"+e.getMessage(),"remove name from file",JOptionPane.ERROR_MESSAGE);
                    }
                }
            });

            //move actions
            actionname = "moveto_"+qf.getFile().getName();
            keyStroke = KeyStroke.getKeyStroke(qf.getKey(),KeyEvent.CTRL_MASK);
            inputMap.put(keyStroke,actionname);
            actionMap.put(actionname,new AbstractAction() {
                public void actionPerformed(ActionEvent event) {
                    try {
                        final String filename = imageFiles.get(index).getName();
                        int linesRemoved =0,filesEdited=0;
                        for(QuickFiles tf1:QuickFiles.values()){
                            int lr = tf1.removeName(filename);
                            if(lr>0){
                                linesRemoved+=lr; filesEdited++;
                            }
                        }
                        qf.addName(filename);
                        setTitle(getDefaultTitle()+" moved to " +
                                qf.getFile().getName()
                                +" (-"+linesRemoved+" lines from "+filesEdited+" files) "
                                +df.format(new Date()));
                    } catch (IOException e) {
                        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                        JOptionPane.showMessageDialog(ImageMonkey.this,"cannot move name to file: \n"+e.getMessage(),"remove name from file",JOptionPane.ERROR_MESSAGE);
                    }
                }
            });
        }
    }

    private final DateFormat df = new SimpleDateFormat("HH:mm ss");

    private void showImage(int i, float zoomFactor) {
        String title=getTitle();
        try {
            final File f = imageFiles.get(i);
            setTitle("loading "+f.getName()+"...");
            setImage(f, zoomFactor,i);
            this.index=i;
            this.zoomFactor=zoomFactor;
            setTitle(getDefaultTitle()+(zoomFactor!=1?" zoom="+String.valueOf(zoomFactor):""));
        } catch (IOException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            setTitle(title);
            JOptionPane.showMessageDialog(this,"cannot show image: \n"+e.getMessage(),"show image",JOptionPane.ERROR_MESSAGE);
        }
    }

    private String getDefaultTitle() {
        return imageFiles.get(index).getName()+" ("+(index+1)+" /"+imageFiles.size()+")";
    }

    private void setImage(File file, float zoomFactor, int index) throws IOException {
        TaskTimer tt = new TaskTimer();
        String fileName=file.getName();
        log.info("load image from " + fileName);
        final PlanarImage img;
        if (zoomFactor!=1 || !isCached(file)) {
            img = fetchScaledImage(file, zoomFactor);

        }
        else{
            log.info("found image in cache");
            img=fetchFromCache(file);
        }
        setImage(img);
        if (zoomFactor==1) { //only cache if no zoomfactor
            enCache(img, file,index);
        }
        log.info(tt.stopString()+" to display "+fileName);
    }


    private PlanarImage fetchScaledImage(File file, float zoomFactor) throws IOException {
        PlanarImage img;
        img = JAI.create("fileload", file.getCanonicalPath());
        if (zoomFactor>0) {
            img=io.scale(img, (int)zoomFactor*getContentPane().getWidth()-10, (int)zoomFactor*getContentPane().getHeight()-10, true);
        }
        return img;
    }

    private void setImage(PlanarImage pi) {
        // h & w need to be height and width minus chrome
        //scale to fit window
        //zoomfactor is a multiple of the window size e.g. *2, *4

        displayPanel.set(pi);
    }
}

