package uk.co.hcsoftware.images.gui.map;

import java.awt.*;


public class GridRef{
    private OsGrid.Letter l;
    private int xOff,yOff;

    /**
     *
     * @param l  letter-square in which this gridref falls
     * @param xOff right-offset in metres from bottom-left of letter square
     * @param yOff up-offset in metres from bottom-left of letter square
     */
    public GridRef(OsGrid.Letter l, int xOff, int yOff) {
        this.l = l;
        this.xOff = xOff;
        this.yOff = yOff;
    }

    private String truncate(int metres, int figures){
        String m = String.valueOf(metres);
        if(m.length()<figures){
            StringBuffer sb = new StringBuffer(figures);
            sb.append(m);
            while(sb.length()<figures) sb.insert(0,"0");
            return sb.toString();
        }
        return m.substring(0,figures);
    }
    /**
     *
     * @param precisionFigures 3 is a 6figure grid
     * @return
     */
    public String getValue(int precisionFigures){



        // express each off as the nearest number that will fit in this number of figures
        //1 means units of 100m
        //2 means units of 10m
        //3 means units of 1m
        //4 means units of 1/10m

        //round each off down to the nearest 3-figure number
        int x = xOff;
        int y = OsGrid.LETTERWIDTH_METRES-yOff; //convert y to bl-relative
        switch(precisionFigures){
            case 1: x/=100; y/=100; break;
            case 2: x/=10; y/=10; break;
            case 3: break;
            case 4: x*=10; y*=10; break;
            default: throw new IllegalArgumentException("invalid precision="+precisionFigures+" must b >=1 <=4");
        }

//        System.out.println("x = " + x);
//        System.out.println("y = " + y);

        x=roundToFigures(x,getMax(precisionFigures));
        y=roundToFigures(y,getMax(precisionFigures));
        return l.getName()+truncate(x,precisionFigures)+truncate(y,precisionFigures);
    }

    private int getMax(int precisionFigures) {
        StringBuffer sb = new StringBuffer();
        for(int i=0;i<precisionFigures;i++){
            sb.append('9');
        }
        return Integer.parseInt(sb.toString());
    }

    private int roundToFigures(int in, int maxSize){
        return Math.min(in,maxSize);
    }

    public String toString() {
        return l.getName()+ "("+xOff+","+yOff+")";
    }


    public Point getPoint() {
        return new Point(l.getTopLeftMetres().x+xOff,l.getTopLeftMetres().y+yOff);
    }
}