package uk.co.hcsoftware.images.gui.map;

import java.io.File;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.FileReader;
import java.awt.*;
import java.util.Map;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * reads postcode-gridref file in format:
 * postcode,x,y
 * on value per line
 *
 * @author spurlinn
 */
public class PostcodeConverter {
    private static final Logger log = Logger.getLogger(PostcodeConverter.class.getName());
    private Map<String,Point> postcodes= new HashMap<String, Point>();
    public PostcodeConverter(File postcodeFile) throws IOException {
        LineNumberReader lnr = new LineNumberReader(new FileReader(postcodeFile));
        try {
            String line;
            while((line=lnr.readLine())!=null){
                try {
                    String[] fields = line.split( "," );
                    int x=Integer.parseInt(fields[1]);
                    int y=Integer.parseInt(fields[2]);
                    //log.log(Level.INFO,gc.getOsGrid(x*10,y*10));
                    postcodes.put(fields[0].trim().toUpperCase(), new Point(x,y));
                } catch (NumberFormatException e) {
                    if(lnr.getLineNumber()!=1 )
                        log.log(Level.WARNING,"invalid line="+lnr.getLineNumber()+" in postcode file="+postcodeFile
                        +" couldn't convert to number");
                } catch (ArrayIndexOutOfBoundsException e) {
                    log.log(Level.WARNING,"invalid line="+lnr.getLineNumber()+" in postcode file="+postcodeFile
                        +" not enough fields");
                }
            }
        } finally {
            lnr.close();
        }
    }
    public Point getXyMetres(String postcode) {
        //File postcodefile=new File(System.getProperty("user.home"),".gridconverter_postcodes");
        return postcodes.get(postcode);
    }
}
