package uk.co.hcsoftware.images.gui.map;

import java.awt.*;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.io.File;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.FileReader;

/**
 * resolves various kinds of map reference into x,y coordinates in metres from top left of UK
 */
public class RefResolver {
    private static final Logger log = Logger.getLogger(RefResolver.class.getName());
    private OsGrid grid = new OsGrid();
    private String pfile;
    private String nfile;


    public RefResolver(String pfile, String nfile) {
        this.pfile = pfile;
        this.nfile=nfile;
    }

    public Point getPoint(String a) throws CannotResolveReferenceException, MapFactory.BadReferenceException {
        //first look in the namefile if it exists
        Point p1 = null;

        try {
            p1=getPointForName(a);
        } catch (CannotGetNamesException e) {
            log.log(Level.WARNING,"couldn't get names",e);
        }
        if(p1!=null){
            log.log(Level.INFO,"found entry in name file="+p1);
            return p1;
        }
        else if(a.indexOf(',')!=-1){ //it's a x,y relative to bottomleft
            try {
                log.log(Level.INFO,a+" is x,y coordinate");
                String[] xy = a.split(",");
                int x=Integer.parseInt(xy[0]);
                int y=Integer.parseInt(xy[1]);
                return new Point(x,y);
            } catch (NumberFormatException e) {
                throw new MapFactory.BadReferenceException(a,"cannot extract numbers from  x,y co-ord");
            } catch(ArrayIndexOutOfBoundsException e){
                throw new MapFactory.BadReferenceException(a,"cannot extract both components from  x,y co-ord");
            }
        }
        else if(a.length()==8){ //it's a landranger
            log.log(Level.INFO,a+" is landranger gridref");
            Point p = decodeLandranger(a);
            return p;
//            GridRef gr=grid.getPointMetresTlRelative(a);
//            if(gr==null) throw new CannotResolveReferenceException(a,"not a valid landranger square");
//            return gr.getPoint();
        }
        else{ //it's a postcode
            try {
                log.log(Level.INFO,a+" is a postcode");
                Point p = getPostcodeConverter().getXyMetres(a);
                if(p==null){
                    throw new CannotResolveReferenceException(a,"uncrecognized postcode");
                }
                return p;
            } catch (CannotGetPostcodesException e) {
                throw new CannotResolveReferenceException(a,"cannot get postcodes",e);
            }
        }
    }

    private Point decodeLandranger(String gridRef) throws CannotResolveReferenceException {
        //frist 2 letters are teh name of the square
        // if(sixFigureGridRef.length()!=6+2) throw new CannotResolveReferenceException(gridRef,"wrong number of characters");
        try {
            // log.log(Level.INFO,"sixFigureGridRef = " + sixFigureGridRef);
            String let=gridRef.substring(0,2).toUpperCase();
            OsGrid.Letter l = grid.getLetter(let);
            int figures = (gridRef.length()-2)/2;
            String x=gridRef.substring(2,2+figures);
            int xValue=Integer.parseInt(x);
            String y=gridRef.substring(2+figures);
            int yValue=Integer.parseInt(y);
            //x and y are relative to bl of square
            //the units of x and y depends on the number of figures in the grid ref
            // - so convert to metres
//            log.log(Level.INFO,"xValue = " + xValue);
//            log.log(Level.INFO,"yValue = " + yValue);
            float m;
            switch(figures){
                case 1: m=10000; break;
                case 2: m=1000; break;
                case 3: m=100; break;
                case 4: m=10f; break;
                default: throw new CannotResolveReferenceException(gridRef,"invalid precision="+figures+" must b >=1 <=4");
            }
            final int xMetres=(int) (xValue*m),yMetres=(int) (yValue*m);

            //return new GridRef(l,xValue,yValue);
            //make metres absolute rather than relative to sqaure
            final Point point = new Point(xMetres+l.x, yMetres+l.y);
            //      log.log(Level.INFO,"point = " + point);
            return point;

        } catch (Exception e) {
            throw new CannotResolveReferenceException(gridRef,"unparseable",e);
        }


    }

    public java.util.List<Point> getPoints(String[] arguments) throws CannotResolveReferenceException, MapFactory.BadReferenceException{
        java.util.List<Point> l = new ArrayList<Point>(arguments.length);
        for (int i = 0; i < arguments.length; i++) {
        }
        return l;
    }


    PostcodeConverter pc=null;
    private PostcodeConverter getPostcodeConverter() throws CannotGetPostcodesException {
        if (pc==null) {
            try {
                /* if(!cmd.hasOption(postcodefile.getOpt())){
                throw new CannotGetPostcodesException("no -p option supplied to lookup postcode");
                }
                String pfile = cmd.getOptionValue(postcodefile.getOpt());*/
                if(pfile==null){
                    throw new CannotGetPostcodesException("cannot find postcodefile name to lookup postcode");
                }
                File f = new File(pfile);
                if(!f.exists()){
                    throw new CannotGetPostcodesException("postcodefile doesn't exist");
                }
                pc= new PostcodeConverter(f);
            } catch (IOException e) {
                log.log(Level.WARNING,"couldn't read postcodes file",e);
                throw new CannotGetPostcodesException("couldn't read postcodes file");
            }
        }
        return pc;
    }
    static class CannotGetPostcodesException extends Exception{
        public CannotGetPostcodesException(String message) {
            super(message);
        }
    }
    static class CannotGetNamesException extends Exception{
        public CannotGetNamesException(String message, Throwable cause) {
            super(message, cause);
        }
    }


    Map<String,Point> nameRefs = null;
    private Point getPointForName(String name) throws CannotGetNamesException {
        if(nameRefs==null){
            try {
                //  String nfile=cmd.getOptionValue(namefile.getOpt());
                if(nfile==null) return null;
                File f = new File(nfile);

                LineNumberReader lnr = new LineNumberReader(new FileReader(f));
                try {
                    nameRefs = new HashMap<String, Point>();
                    String line;
                    while((line=lnr.readLine())!=null){
                        if (line.indexOf(":")!=-1) {
                            try {
                                String[] fields = line.split( ":" );
                                String place=fields[0].trim().toLowerCase();
                                if(place.equals("root")) continue;
                                String value=fields[1].trim().toLowerCase();
                                //log.log(Level.INFO,gc.getOsGrid(x*10,y*10));
                                try {
                                    place=place.replaceAll(" ","_");
                                    nameRefs.put(place,decodeLandranger(value));
                                } catch (CannotResolveReferenceException e) {
                                        log.log(Level.WARNING,"invalid line="+lnr.getLineNumber()+" in placenamefile file="+f
                                                +" can't convert lr="+value+" to x,y",e);
                                }
                            } catch (NumberFormatException e) {
                                log.log(Level.WARNING,"invalid line="+lnr.getLineNumber()+" in placenamefile file="+f
                                            +" couldn't convert to number");
                            } catch (ArrayIndexOutOfBoundsException e) {
                                log.log(Level.WARNING,"invalid line="+lnr.getLineNumber()+" in placenamefile file="+f
                                            +" not enough fields");
                            }
                        }
                    }
                } finally {
                    lnr.close();
                }
            } catch (NullPointerException e) {
                throw new CannotGetNamesException("couldn't get placenamefile from options",e);
            } catch (IOException e) {
                throw new CannotGetNamesException("error reading placenamefile",e);
            }
        }
        return nameRefs.get(name);
    }

}
