package uk.co.hcsoftware.images.tools;

import com.sun.media.jai.codec.ForwardSeekableStream;
import ns.TaskTimer;
import org.apache.commons.cli.*;
import uk.co.hcsoftware.images.UsageException;

import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.media.jai.RenderedOp;
import java.awt.*;
import java.awt.image.renderable.ParameterBlock;
import java.io.File;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Chop {
    public static final Logger log = Logger.getLogger(Chop.class.getName());
    /*
    a4 dimensions in cm = 210x297
                  in in = 8.25x11.75
    */

    private static final Options availableOptions = new Options();
    private static CommandLine cmd;
    public static final Option width = new Option("w",true,"width in pixels ");
    public static final Option height = new Option("h",true,"height in pixels");
    public static final Option prefix = new Option("s",true,"filename pattern e.g. /tmp/image.png will create /tmp/image1.png /tmp/image2.png etc.");


    static{
        prefix.setRequired(true);
        try {
            Field[] fields = Chop.class.getDeclaredFields();
            for (int i = 0; i < fields.length; i++) {
                Field field = fields[i];
                if(field.getType()==Option.class){ //all fields that are Option s
                    Chop.availableOptions.addOption((Option)field.get(null));
                }
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

    }

    private static void usage(){
        HelpFormatter formatter = new HelpFormatter();
        formatter.printHelp(80
                , Chop.class.getName()+" [-width n] [-height n] -s nameprefix [imagedata | imagefilename]"
                ,"chop the input image up into images each width by height pixels in size. Save to files"
                , availableOptions
                ,"Example:\n  java -jar "+Chop.class.getName().substring(Chop.class.getName().lastIndexOf('.')+1)
                +".jar -width 2000 -height 2000 -s /tmp/image myimage.png-\n" +
                ""
                ,false
        );
    }

    public static void main(String[] args) {
        try {
           // System.out.println("availableOptions = " + availableOptions);
            cmd = new GnuParser().parse(availableOptions,args);
            if(log.isLoggable(Level.INFO)){
                Option[] opts = cmd.getOptions();
                StringBuffer sb = new StringBuffer();
                for (int i = 0; i < opts.length; i++) {
                    Option opt = opts[i];
                    sb.append(opt.getOpt()+"="+opt.getValue());
                    if(i+1<opts.length) sb.append(" ");
                }
                log.log(Level.INFO,"options: "+sb.toString());
            }
            final String[] arguments = cmd.getArgs();

            //get the outfileprefix
            final String outfileprefix;
            final File outdir;
            String ofw=Chop.cmd.getOptionValue(Chop.prefix.getOpt());
            File f = new File(ofw);
            outdir = f.getParentFile();
            outfileprefix = f.getAbsolutePath().substring(outdir.getAbsolutePath().length()+1);//probably includes filesep
            log.log(Level.INFO,"output dir="+outdir+" outputfileprefix="+outfileprefix);
            int wP=-1,hP=-1;
            if(Chop.cmd.hasOption(Chop.width.getOpt()) ){
                String w=Chop.cmd.getOptionValue(Chop.width.getOpt());
                wP=Integer.parseInt(w);
            }
            if(Chop.cmd.hasOption(Chop.height.getOpt()) ){
                String w=Chop.cmd.getOptionValue(Chop.height.getOpt());
                hP=Integer.parseInt(w);
            }
            if(arguments.length>0){
                if(Arrays.asList(new String[]{"--help","help","h","-h"}).contains(args[0])){
                    throw new UsageException("invalid args");
                }
                List<String> fnames= Arrays.asList(arguments);
                for(String fi: fnames){
                    if(!new File(fi).exists()){
                        throw new UsageException("file="+f+" not found!");
                    }
                }
                new Chop(wP,hP,outdir,outfileprefix).run(fnames);
                return;
            }
            else{
                new Chop(wP,hP,outdir,outfileprefix).run(System.in);
                return;
            }
        }  catch(MissingOptionException e){
            System.err.println("Missing option(s): "+e.getMessage());
        } catch (ParseException e) {
            e.printStackTrace();
        }  catch (UsageException e) {
            System.err.println("invalid usage: "+e.getMessage());
        }
        usage();
    }
    private int wPixels, hPixels;
    File outdir;
    String filePrefix;

    public Chop(int wPixels, int hPixels, File outdir, String filePrefix) {
        this.wPixels = wPixels;
        this.hPixels = hPixels;
        this.outdir = outdir;
        this.filePrefix = filePrefix;
    }

    private void chop(PlanarImage pi){
        TaskTimer tt = new TaskTimer();
        ImageOperations io = new ImageOperations();
        if(wPixels==-1) wPixels=pi.getWidth();
        if(hPixels==-1) hPixels=pi.getHeight();
        log.log(Level.INFO,"chop image "+j+" wpixels="+wPixels+" hPixels="+hPixels);
        List<PlanarImage> pis = io.chop(pi,wPixels,hPixels,Color.white);
        //save the images
        for (PlanarImage pii:pis) {
            save(pii);
        }
        log.log(Level.INFO,tt.stopString()+" to chop image into "+pis.size()+" tiles");
    }
    private void run(InputStream is) {
        //first load the image
        final ForwardSeekableStream forwardSeekableStream = new ForwardSeekableStream(is);
        final RenderedOp pi = JAI.create("stream", forwardSeekableStream);
        chop(pi);
    }
    private void run(List<String> filenames) {
        //first load the image
        for(String f:filenames){
            log.log(Level.INFO,"loading image from "+f);
            final RenderedOp pi = JAI.create("fileload", f);
            chop(pi);
        }
    }
    private int j=0;
    private void save(PlanarImage pi) {
        String savefile=filePrefix;
        String ext="";
        String allButExt=savefile;
        int i=savefile.indexOf('.');//= savefile.substring()
        if(i!=-1){
            ext=savefile.substring(i);
            allButExt=savefile.substring(0,i);
        }
        savefile=allButExt+j+ext;
        File f = new File(outdir,savefile);
        log.log(Level.INFO,"saving to "+f.getAbsolutePath());

        ParameterBlock pb = new ParameterBlock();
        pb.addSource(pi);
        pb.add(f.getAbsolutePath());

        //  final String of = cmd.getOptionValue(outformat.getOpt(),"png");
        //  pb.add(of);
        JAI.create("filestore", pb, null);
        j++;
    }

}
