package uk.co.hcsoftware.images.tools;

import javax.media.jai.*;
import javax.media.jai.widget.ScrollingImagePanel;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.ColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.SampleModel;
import java.awt.image.renderable.ParameterBlock;
import java.util.ArrayList;
import java.util.List;


public class ImageOperations {
    public PlanarImage scale(PlanarImage pi, int maxOutputWidth, int maxOutputHeight, boolean preserveAspectRatio){
        return scale(pi,maxOutputWidth,maxOutputHeight,preserveAspectRatio,Interpolation.INTERP_BILINEAR);
    }

    /**
     *  if preserveAspectRatio is true, then either maxOutputWidth or maxOutputHeight can be -1 in which case the parameter
     * will be ignored
     * @param pi
     * @param maxOutputWidth
     * @param maxOutputHeight
     * @param preserveAspectRatio
     * @return
     */
    public PlanarImage scale(PlanarImage pi, int maxOutputWidth, int maxOutputHeight, boolean preserveAspectRatio,int interpType){
        int startWidth=pi.getWidth();
        int startHeight=pi.getHeight();
        if(startWidth==maxOutputWidth && startHeight<=maxOutputHeight) return pi;
        if(startHeight==maxOutputHeight && startWidth<=maxOutputWidth) return pi;


       // log.log(Level.INFO,"input w="+startWidth+" h="+startHeight);
        float xScaleFactor=(float)maxOutputWidth/(float)startWidth;
        float yScaleFactor=(float)maxOutputHeight/(float)startHeight;
        //preserve the aspect ratio.. i.e. make sure x and y are the same

        if(preserveAspectRatio && xScaleFactor!=yScaleFactor){
            if(maxOutputWidth==-1 && maxOutputHeight!=-1){
                xScaleFactor=yScaleFactor;
            }
            else if(maxOutputWidth!=-1 && maxOutputHeight==-1){
                // xScaleFactor=xScaleFactor;
            }
            else{
                //whichever is the smaller
                xScaleFactor=Math.min(xScaleFactor,yScaleFactor);
            }
            yScaleFactor=xScaleFactor;
        }
     /*   log.log(Level.INFO,"xScaleFactor = " + xScaleFactor);
        log.log(Level.INFO,"yScaleFactor = " + yScaleFactor);
*/
        /*
        * Create a standard bilinear interpolation object to be
        * used with the "scale" operator.
        */
        Interpolation interp = Interpolation.getInstance(interpType);

        /**
         * Stores the required input source and parameters in a
         * ParameterBlock to be sent to the operation registry,
         * and eventually to the "scale" operator.
         */
        ParameterBlock params = new ParameterBlock();
        params.addSource(pi);
        params.add(xScaleFactor);         // x scale factor
        params.add(yScaleFactor);         // y scale factor
        params.add(0.0F);         // x translate
        params.add(0.0F);         // y translate
        params.add(interp);       // interpolation method

        /* Create an operator to scale image1. */
        return JAI.create("scale", params);
    }

    public PlanarImage rotate(PlanarImage pi, float angle){
        //find the centre
        float x=(float)pi.getWidth()/2f;
        float y=(float)pi.getHeight()/2f;
        /**
         * Stores the required input source and parameters in a
         * ParameterBlock to be sent to the operation registry,
         * and eventually to the "scale" operator.
         */
        ParameterBlock params = new ParameterBlock();
        params.addSource(pi);
        params.add(x);         // x origin
        params.add(y);         // y origin
        params.add(angle);         // angle

        /* Create an operator to scale image1. */
        return JAI.create("rotate", params);
    }


     private PlanarImage watermark(PlanarImage pi, String text){
       //scale the watermark to the same size as the images
        //watermark=scale(watermark,pi.getWidth(),pi.getHeight(),true);

         // Create the ParameterBlock.
         /*Byte[] bandValues = new Byte[1];
         bandValues[0] = Byte.parseByte("0");
         ParameterBlock pb = new ParameterBlock();
         pb.add(new Float(pi.getWidth()));   // The width
         pb.add(new Float(pi.getHeight()));  // The height
         pb.add(bandValues);                   // The band values

         // Create the constant operation.
         PlanarImage piw = (PlanarImage)JAI.create("constant", pb);*/
         int w=pi.getWidth(),h=pi.getHeight();
         // Create a float data sample model.
    SampleModel sampleModel =
       RasterFactory.createBandedSampleModel(DataBuffer.TYPE_INT,
                                             w,
                                             h,
                                             1);
    // Create a compatible ColorModel.
    ColorModel colorModel = PlanarImage.createColorModel(sampleModel);
         TiledImage piw = new TiledImage(w,h,w,h,0,0,sampleModel,colorModel);
         Graphics2D g2 = (Graphics2D) piw.getGraphics();
         g2.setColor(Color.BLACK);
         g2.drawString(text,0,0);

         //composite the 2
         return piw;

   }

    /**
     * chop the input image up into separate images each wPixels by hPixels, moving l-r top-bottom across the input image
     * @param input
     * @param wPixels
     * @param hPixels
     * @param backColor can be null. If not fill all empty regions with this colour
     * @return
     */
    public List<PlanarImage> chop(PlanarImage input, int wPixels, int hPixels, Color backColor){
        //this is probably very inefficient.. but easy to code!
        List<PlanarImage> l = new ArrayList<PlanarImage>();
        int x,y;
        for(y=0;y<input.getHeight();y+=hPixels){
            for(x=0;x<input.getWidth();x+=wPixels){
                TiledImage ti = new TiledImage(0,0,wPixels,hPixels,0,0,input.getSampleModel(),input.getColorModel());
                Graphics2D g = ti.createGraphics();
                if (backColor!=null) {
                    g.setColor(backColor);
                    g.fillRect(0,0,wPixels,hPixels);
                }
                AffineTransform a = new AffineTransform();
                a.setToTranslation(-x,-y);
                g.drawRenderedImage(input,a);
                l.add(ti);
            }
        }

        return l;
    }

    public static void main(String[] args) {
        PlanarImage pi = JAI.create("fileload", args[0]);
        final ImageOperations iop = new ImageOperations();
        pi=iop.watermark(pi,"hello");
        Frame f = new Frame();
        // Display the original in a scrolling window
        ScrollingImagePanel imagePanel1 = new ScrollingImagePanel(pi, 100, 100);

        // Add the display widget to our frame.
        f.add(imagePanel1);
        f.setSize(200,200);
        f.show();

    }

    public Image scale(Image image, int startWidth, int startHeight, int maxOutputWidth, int maxOutputHeight, boolean preserveAspectRatio, int scaleType){
        if(startWidth==maxOutputWidth && startHeight<=maxOutputHeight) return image;
        if(startHeight==maxOutputHeight && startWidth<=maxOutputWidth) return image;

        float xScaleFactor=(float)maxOutputWidth/(float)startWidth;
        float yScaleFactor=(float)maxOutputHeight/(float)startHeight;
        //preserve the aspect ratio.. i.e. make sure x and y are scaled the same

        if(preserveAspectRatio && xScaleFactor!=yScaleFactor){
            if(maxOutputWidth==-1 && maxOutputHeight!=-1){
                xScaleFactor=yScaleFactor;
            }
            else if(maxOutputWidth!=-1 && maxOutputHeight==-1){
                // xScaleFactor=xScaleFactor;
            }
            else{
                //whichever is the smaller
                xScaleFactor=Math.min(xScaleFactor,yScaleFactor);
            }
            yScaleFactor=xScaleFactor;
        }
        int drawW=(int) (xScaleFactor*startWidth);
        int drawH=(int) (yScaleFactor*startHeight);
        return image.getScaledInstance(drawW,drawH,scaleType);
    }
}
