package uk.co.hcsoftware.images.tools.cli;

import ns.TaskTimer;
import uk.co.hcsoftware.images.tools.ImageOperations;
import uk.co.hcsoftware.yago.Option;
import uk.co.hcsoftware.yago.parsers.GnuParser;
import uk.co.hcsoftware.yago.util.DefaultArgReaders;

import javax.media.jai.JAI;
import javax.media.jai.ParameterBlockJAI;
import javax.media.jai.PlanarImage;
import java.awt.image.renderable.ParameterBlock;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Title: Scale
 * Copyright:    Copyright (c) 20-Dec-2005
 *
 * @author spurlinn
 */
public class Scale {
    public static final Logger log = Logger.getLogger(Scale.class.getName());
    Option<File> in = new Option<File>("i",true,DefaultArgReaders.createFileReader(true),"input imagefile");
    Option<File> out = new Option<File>("o",true,DefaultArgReaders.createFileReader(false),"output imagefile");

    Option<Integer> maxw = new Option<Integer>("maxw",true,DefaultArgReaders.createIntReader(),"max output width");
    Option<Integer> maxh = new Option<Integer>("maxh",true,DefaultArgReaders.createIntReader(),"max output height");
    Option<Boolean> ignoreAspect=new Option<Boolean>("ignoreaspect",false,"don't preserve aspect ratio (default preserves)");

    Option<Integer> presmooth = new Option<Integer>("presmooth",false,DefaultArgReaders.createIntReader(),"pre smoothing 1-10");
    Option<Integer> postsharpen = new Option<Integer>("postsharpen",false,DefaultArgReaders.createIntReader(),"post sharpening 1-20");

     public static void main(String[] args) {
         new Scale().run(args);
     }

    private void run(String[] args) {
        GnuParser gp = new GnuParser(in,out,maxw,maxh,ignoreAspect,presmooth,postsharpen);
        gp.createBundleOptionGroup(maxw,maxh);
        gp.parseOrDie(Scale.class.getName(),args,"scales input image");

        try {
            TaskTimer tt = new TaskTimer();
           // JAI.
            PlanarImage pi =  JAI.create("fileload", in.getArgValue().getCanonicalPath());
            if(presmooth.isSet()){
                final float amount = -(presmooth.getArgValue()/10f);
                log.info("presmoothing: unsharpmask="+amount);
                pi = unsharpMask(pi,amount);
            }
            pi =new ImageOperations().scale(pi,maxw.getArgValue(),maxh.getArgValue(),!ignoreAspect.isSet());
            if(postsharpen.isSet()){
                final float amount = (postsharpen.getArgValue()/20f);
                log.info("postsharpening: unsharpmask="+amount);
                pi=unsharpMask(pi,amount);
            }
            ParameterBlock pb = new ParameterBlock();
            pb.addSource(pi);
            final File outFile = out.getArgValue();
            pb.add(outFile.getCanonicalPath());
            pb.add(null);
            pb.add(null);
            JAI.create("filestore", pb, null);
            log.info("wrote file= " + outFile+ " took "+tt.stopString());
        } catch (IOException e) {
            log.log(Level.WARNING, "exception", e);
        }
    }
    private PlanarImage unsharpMask(PlanarImage pi, final float amount) {
        ParameterBlockJAI pbu = new ParameterBlockJAI("UnsharpMask");
        pbu.addSource(pi);
        pbu.setParameter("gain", amount);
        pi = JAI.create("UnsharpMask", pbu , null);
        return pi;
    }
}
