package uk.co.hcsoftware.images.web;

import java.util.logging.Logger;


/**
 * converts an xml element spec to an outputstring
 * Title: ElementEncoder
 * Copyright:    Copyright (c) 12-Jan-2006
 *
 * @author spurlinn
 */
public class ElementEncoder {
    public static final Logger log = Logger.getLogger(ElementEncoder.class.getName());
    private static ElementEncoder ee;
    public enum ElementType{
        /**
         * e.g. <a/>
         */
        simple
        /**
         * e.g. <a>
         */
        ,complexOpen
        /**
         * e.g. </a>
         */
        , complexClose
    }
    /**
     * generates an XML element fragment of the form <a href="bb"/> or <a> or </a>
     *
     * NB: doesn't do any escaping of attributevalues at present - you are responsible for that yourself!
     * (so you will get invalid XML if your attributeValuePairs contain unescaped chars such as "
     *
     * @param elementName
     * @param et
     * @param attributeValuePairs
     * @return
     */
    public String generateUrl(String elementName, ElementType et, String... attributeValuePairs){
        if(et==ElementType.complexClose){
            return "</"+elementName+">";
        }
        StringBuilder sb = new StringBuilder("<");
        sb.append(elementName);
        //append attributes
        assert attributeValuePairs.length%2==0:"invalid attributeValuePairs.length (should be divisible by 2)!="+attributeValuePairs.length;
        
            for (int i = 0; i < attributeValuePairs.length; i+=2) {
                sb.append(" ");
                sb.append(attributeValuePairs[i]);
                sb.append("=");
                sb.append("\"");
                sb.append(attributeValuePairs[i+1]);
                sb.append("\"");
            }
        if(et==ElementType.simple){
            sb.append("/");
        }
        sb.append(">");
        return sb.toString();
    }
//    /**
//     * convenience method that is not threadsafe
//     * @param elementName
//     * @return
//     */
//    public static String generateUrlStatic(String elementName, ElementType et, String[]... attributeValuePair){
//        if(ee==null) ee = new ElementEncoder();
//        return ee.generateUrl(elementName);
//    }
}
