package uk.co.hcsoftware.images.web;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Comparator;
import java.util.logging.Logger;


public class GalleryIndex {
    final static Logger log = Logger.getLogger(GalleryIndex.class.getName());
    private final static GalleryIndex gi = new GalleryIndex();

    private static final File imageTreeRoot;
    static{
        if(System.getProperty("os.name").toLowerCase().indexOf("win")>=0){
            imageTreeRoot= new File("c:\\temp\\t\\images");
        }
       else imageTreeRoot= new File("/var/images");
    }
    public static GalleryIndex getInstance(){
       // if(gi==null) log.warning("gi is null! this should be impossible!");
        return gi;
    }

    private List<Gallery> galleries;
    /**
     * default implentation uses the filesystem. ImageTree looks like this
     * rootDir
     *   gallery1
     *     thumbs
     *       thumb1
     *       thumb2
     *     fullsize
     *       fullsize1
     *       fullsize2
     *     original1
     *     original2
     *    gallery2
     *      ..
     *
     *
     *      */
    public GalleryIndex() {
    }

    public void buildIndex() {
        final List<Gallery> galleries;
        if(imageTreeRoot.exists()){
            ImagesMetadata imd = new ImagesMetadata(imageTreeRoot);
            File[] galleryDirectories = imageTreeRoot.listFiles(new FileFilter() {
                public boolean accept(File pathname) {
                    return pathname.isDirectory();
                }
            });
            List<Gallery> ret = new ArrayList<Gallery>(galleryDirectories.length);
            for (int i = 0; i < galleryDirectories.length; i++) {
                File f = galleryDirectories[i];
                ret.add(new Gallery(f.getName(),f, imd));
            }
            //sort them by name
            Collections.sort(ret,new Comparator<Gallery>() {
                public int compare(Gallery gallery, Gallery gallery1) {
                    return gallery.getName().compareTo(gallery1.getName());
                }
            });
            galleries= ret;
        }
        else{
            log.warning("imageTreeRoot="+imageTreeRoot+" does not exist! assuming 0 galleries");
            galleries= Collections.emptyList();
        }
        synchronized (GalleryIndex.class) {
            this.galleries=galleries;
        }
    }

    public List<Gallery> getGalleries() {
        synchronized (GalleryIndex.class) {
            if(galleries==null) log.warning("galleries is null!");
            return galleries;
        }
    }
    public Gallery getGallery(final String id) throws NoSuchGalleryException{
        for(Gallery i:galleries){
            if(id.equals(i.getId())){
                return i;
            }
        }
        throw new NoSuchGalleryException(id);
    }


}
