package uk.co.hcsoftware.images.web;

import java.io.File;
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;


public class Image {
    String id;
    File orig;
     File thumb;
    private final ImagesMetadata imd;
     File full;
    private BlurbReader br;
    private ElementEncoder ee=new ElementEncoder();

    public Image(String id, File orig, File thumb, File full, ImagesMetadata imd) {
        this.full = full;
        this.id = id;
        this.orig = orig;
        this.thumb = thumb;
        this.imd = imd;
        br = new BlurbReader(new File(orig.getParentFile(),orig.getName()+".blurb"));
    }


    public File getFull() {
        return full;
    }

    public String getId() {
        return id;
    }

    public File getOrig() {
        return orig;
    }

    public File getThumb() {
        return thumb;
    }

    public String getThumbHtml(Gallery g){
        String imageRef="id="+getId()+"&gallery="+g.getId();
        StringBuilder sb = new StringBuilder();
//        sb.append("<a href=\"view_image.jspx?"+imageRef+"\">");
//        sb.append("<img class=\"thumb\"");
//        sb.append(" src=\"load?thumb=true&"+imageRef+"\"");
//        if(hasBlurb()){
//            String tooltip = truncate(getBlurb(),50);
//            sb.append(" title=\"");
//            sb.append(tooltip);
//            sb.append("\"");
//        }
//        sb.append("/>");
//        sb.append( "</a>");

        //<a ..>
        sb.append(ee.generateUrl("a",ElementEncoder.ElementType.complexOpen,
                "href","view_image.jspx?"+imageRef
        ));
        List<String> imgAttributes = new ArrayList<String>();
        imgAttributes.addAll(Arrays.asList(
                new String[]{
                     "class","thumb"
                     ,"src","load?thumb=true&"+imageRef
                }
        ));
        if(hasBlurb()){
            String tooltip = truncate(getBlurb(),50);
            imgAttributes.add("title");
            imgAttributes.add(tooltip);
        }

        ImagesMetadata.Metadata md = imd.getMetadata(thumb);
        if(md!=null && md.hasWidth() && md.hasHeight()){
            imgAttributes.add("width");
            imgAttributes.add(String.valueOf(md.getW()));

            imgAttributes.add("height");
            imgAttributes.add(String.valueOf(md.getH()));
        }

        //<img ../>
        sb.append(ee.generateUrl("img",ElementEncoder.ElementType.simple
            ,imgAttributes.toArray(new String[imgAttributes.size()])
        ));
        //</a>
        sb.append(ee.generateUrl("a",ElementEncoder.ElementType.complexClose, (String[])null ));

        return sb.toString();
    }

    private String truncate(String s, int maxLen) {
        if(s.length()>maxLen){
            return s.substring(0,maxLen-3)+"...";
        }
        else return s;
    }

    public String getFullsizeHtml(Gallery g){
        String imageref="id="+getId()+"&gallery="+g.getId();
//        return  "<img class=\"fullsize\"" +
//                " src=\"load?"+imageref+"\"" +
//                "/>" ;

        List<String> imgAttributes = new ArrayList<String>();
        imgAttributes.add("class");
        imgAttributes.add("fullsize");
        imgAttributes.add("src");
        imgAttributes.add("load?"+imageref);

        ImagesMetadata.Metadata md = imd.getMetadata(full);
        if(md!=null && md.hasWidth() && md.hasHeight()){
            imgAttributes.add("width");
            imgAttributes.add(String.valueOf(md.getW()));

            imgAttributes.add("height");
            imgAttributes.add(String.valueOf(md.getH()));
        }

        return ee.generateUrl("img",ElementEncoder.ElementType.simple
                ,imgAttributes.toArray(new String[imgAttributes.size()])

//                ,"width", "700"
//                ,"height", "700"
        );
    }

    public boolean hasBlurb(){
        return br.blurbExists();
    }
    public String getBlurb(){
        return br.getBlurb();
    }

}
