package uk.co.hcsoftware.images.web;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.ParserConfigurationException;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.Map;
import java.util.HashMap;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;


/**
 * Title: ImagesMetadata
 * Copyright:    Copyright (c) 12-Jan-2006
 *
 * @author spurlinn
 */
public class ImagesMetadata {
    public static final Logger log = Logger.getLogger(ImagesMetadata.class.getName());
    private Map<File,Metadata> imageFiles=new HashMap<File, Metadata>();
    public static class Metadata{
        private final int w,h;

        public Metadata(int w, int h) {
            this.w = w;
            this.h = h;
        }

        public int getW() {
            return w;
        }

        public int getH() {
            return h;
        }

        public boolean hasHeight() {
            return h>-1;
        }

        public boolean hasWidth() {
            return w>-1;
        }
    }
    public ImagesMetadata(final File rootDir) {
        if(rootDir.isDirectory() && rootDir.exists()){
            File imagesXml = new File(rootDir,"images.xml");
            if(!imagesXml.isFile() || !imagesXml.exists()){
                log.info("images.xml not found or doesn't exist");
                return;
            }
             //parse the file and harvest the metadata
            try {
                SAXParser sp = SAXParserFactory.newInstance().newSAXParser();
                final FileInputStream fis = new FileInputStream(imagesXml);
                try {
                    imageFiles.clear();
                    sp.parse(fis,new DefaultHandler(){
                        public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
                            if("image".equals(qName)){
                                String path = attributes.getValue("path");
                                String width = attributes.getValue("width");
                                String height = attributes.getValue("height");
                                if (path!=null && width!=null && height!=null) {
                                    File f = new File(rootDir,path);
                                    imageFiles.put(f,new Metadata(Integer.parseInt(width),Integer.parseInt(height)));
                                } else {
                                    log.fine("invalid element path="+path+" w="+width+" h="+height);
                                }
                            }
                        }
                    });
                } finally {
                    fis.close();
                }
            } catch (ParserConfigurationException e) {
                log.log(Level.WARNING, "exception", e);
            } catch (SAXException e) {
                log.log(Level.WARNING, "exception", e);
            } catch (IOException e) {
                log.log(Level.WARNING, "exception", e);
            }

        }
        else log.info("images.xml not found or not a file");
    }
    public Metadata getMetadata(File imageFile){
        return imageFiles.get(imageFile);
    }


}
